clear
saving_image = 0;
show_stage = 1;
[final_cell_num,~,sim_name,division_time,~,~,~,~,~,~,~,~]=model_para(0);

%% load trajectory and contact map
file_list = string(ls);
load(file_list(contains(file_list,'tory.mat')))
if show_stage == 1
	contmap_file_name = file_list(contains(file_list,'p.mat'));
	if isempty(contmap_file_name)
		warning('no contact_map file')
		show_stage = 0;
	else
		load(contmap_file_name)
		[unique_adjm_stack,adjm_cyc] = contact_map_pathway(contact_map_batch);
		save_size = cellfun(@length,unique_adjm_stack) == final_cell_num;
		show_idx = find(save_size);
		show_num = sum(save_size);
		c = jet(show_num);
	end
end
% c = [jet(4);[0.5,0,1];[0.5,0.5,1]];% 9_2_33
if exist('init_stage','var')
	[v_mean,v_extreme,dvdt_extreme,time_seq] = dynamics_analysis(trajectory,division_time,'initial cyc',init_stage);
else
	[v_mean,v_extreme,dvdt_extreme,time_seq] = dynamics_analysis(trajectory,division_time);
end
dvdt = diff(v_mean);

% range
e_max = 1.2e-1;
if isempty(dvdt_extreme)
	min_dvdt = max(1.05*floor(min(dvdt)/1e-4)*1e-4,-2e-3);
	max_dvdt = min(1.05*floor(max(dvdt)/1e-4)*1e-4,2e-3);
else
	min_dvdt = max(1.05*floor(min(dvdt(dvdt_extreme))/1e-4)*1e-4,-2e-3);
	max_dvdt = min(1.05*ceil(max(dvdt(dvdt_extreme))/1e-4)*1e-4,2e-3);
end
min_dvdt(min_dvdt>-0.001) = -0.001;
max_dvdt(max_dvdt<0.001) = 0.001;

%% plotting
figure;set(gcf,'unit','normalized','position',[0,0,1,0.7]);
f = 1:4;
ax1 = axes('position',[0.07,0.575,0.9,0.4]);hold on
ax2 = axes('position',[0.07,0.125,0.9,0.4]);hold on
% contact map stage
if show_stage == 1
	for n = 1:show_num
		stage_cyc = sort(adjm_cyc{show_idx(n)});
		break_idx = [find([false;stage_cyc(2:end)-stage_cyc(1:end-1)>1000]);length(stage_cyc)];
		lower_x = min(stage_cyc);
% 		upper_x = max(stage_cyc);
		for m = 1:length(break_idx)
			if break_idx(m)<length(stage_cyc)
				upper_x = stage_cyc(break_idx(m)-1);
			else
				upper_x = stage_cyc(break_idx(m));
			end
			if upper_x == lower_x
				upper_x = upper_x+50;
			end
			v1 = [[lower_x;upper_x;upper_x;lower_x],[0;0;e_max;e_max]];
			v2 = [[lower_x;upper_x;upper_x;lower_x],[min_dvdt;min_dvdt;max_dvdt;max_dvdt]];
			patch(ax1,'Faces',f,'Vertices',v1,'FaceColor',c(n,:),'EdgeColor','none','FaceAlpha',0.3)
			patch(ax2,'Faces',f,'Vertices',v2,'FaceColor',c(n,:),'EdgeColor','none','FaceAlpha',0.3)
			lower_x = stage_cyc(break_idx(m));
		end
	end
end
% critical cyc
for n = 1:length(v_extreme)
	plot(ax1,time_seq([v_extreme(n),v_extreme(n)]),[0,e_max],'k','linewidth',1.5)
	plot(ax2,time_seq([v_extreme(n),v_extreme(n)]),[min_dvdt,max_dvdt],'k','linewidth',1.5)
end
for n = 1:length(dvdt_extreme)
	plot(ax1,time_seq([dvdt_extreme(n),dvdt_extreme(n)]),[0,e_max],'k--','linewidth',1.5)
	plot(ax2,time_seq([dvdt_extreme(n),dvdt_extreme(n)]),[min_dvdt,max_dvdt],'k--','linewidth',1.5)
end
% v-t
plot(ax1,time_seq(2:end),v_mean(1:length(time_seq)-1),'k','linewidth',2);
xlim(ax1,[time_seq(1),time_seq(end)])
ylim(ax1,[0,e_max])
set(ax1,'FontSize',20,'Fontname','arial','xticklabels',[],'ytick',[0,0.05,0.1],'YTickLabel',{'0','0.05','0.10'},'ticklength',[5e-3,0])
% dvdt-t
plot(ax2,time_seq(5:end),zeros(size(time_seq(3:end-2))),'--','Color',0.6*ones(1,3),'linewidth',1.5)
plot(ax2,time_seq(5:end),dvdt(3:length(time_seq)-2),'k','linewidth',2)
xlim(ax2,[time_seq(1),time_seq(end)])
ylim(ax2,[min_dvdt,max_dvdt])
set(ax2,'FontSize',20,'Fontname','arial','ticklength',[5e-3,0])
set(ax2,'YTickLabel',arrayfun(@(x)keep_decimal(x,3),get(ax2,'ytick'),'UniformOutput',false))

%% disp download info
command = [];
download_cyc = unique(sort(time_seq([v_extreme,dvdt_extreme])));
for n = 1:length(download_cyc)
	command = [command,num2str(download_cyc(n)),',']; %#ok<AGROW>
end
disp(['*_{',command(1:end-1),'}.mat'])

%% save dynamics curve
if saving_image == 1
	img_name = sprintf('%dcell%s_dynamics',final_cell_num,sim_name(find(sim_name == '_', 1, 'last' ):end));
	if exist(sprintf('%s.jpg',img_name),'file') == 2
		user_resp = input('dynamics curve exists. Replace? y/n\n','s');
		if user_resp == 'y'
			print(img_name,'-djpeg','-r600')
		end
	else
		print(img_name,'-djpeg','-r600')
	end
end