function [v_mean,v_extreme,dvdt_extreme,time_seq] = dynamics_analysis(varargin)
trajectory = varargin{1};
division_time = varargin{2};
specify_init_stage = 0;
specify_final_cyc = 0;
for str_idx = find(cellfun(@ischar,varargin))
	switch varargin{str_idx}
		case 'initial cyc'
			init_cyc = ceil((varargin{str_idx+1}+1)/100)*100;
			specify_init_stage = 1;
		case 'final cyc'
			final_cyc = varargin{str_idx+1};
			specify_final_cyc = 1;
	end
end

%% time
stage = size(trajectory,2);
stage_length = cellfun(@length,trajectory(:,stage));
N = min(stage_length(stage_length>0));
N_max = max(cellfun(@length,trajectory(:,stage)));
min_init_cyc = max(division_time(:,stage-1));
if specify_init_stage == 0
	init_cyc = min_init_cyc;
elseif specify_init_stage == 1 && init_cyc<min_init_cyc
	warning('initial cyc setting error')
	init_cyc = min_init_cyc;
end
max_final_cyc = init_cyc+(N_max-1)*100;
if specify_final_cyc == 0
	final_cyc = max_final_cyc;
elseif specify_final_cyc == 1 && final_cyc>max_final_cyc
	warning('final cyc setting error')
	final_cyc = max_final_cyc;
end
time_seq = init_cyc:100:final_cyc;

%% calculate v and dvdt
v_square = zeros(size(trajectory,1),N-1);
for n = find(stage_length>0)'
	vsq = sum(diff(trajectory{n,stage}).^2,2);
	v_square(n,:) = vsq(end-N+2:end);
end
v_mean = sqrt(sum(v_square)/size(trajectory,1));
if length(v_mean)>length(time_seq)
	v_mean = v_mean(1:length(time_seq));
end
dvdt = diff(v_mean);

%% localmin & localmax v
[~,max_loc] = findpeaks(v_mean);
[~,min_loc] = findpeaks(-v_mean);
v_extreme = [min_loc(min_loc>4),max_loc(max_loc>4)]+1;
if v_mean(end)<v_mean(end-1)&&v_mean(end)<1e-1
	v_extreme = [v_extreme,length(time_seq)];
end

%% localmin & localmax dvdt
[~,localmax_loc] = findpeaks(dvdt);
[~,localmin_loc] = findpeaks(-dvdt);
dvdt_extreme = [localmax_loc(localmax_loc>3),localmin_loc(localmin_loc>3)]+2;