% sim config
% N = 256;
% analysis_step = 100;
% phi_save_step = 100;
% phi_save_switch = 1;
% trajectory_save = 1;
% dynamics_analyse = 0;
% compressed = 1;
% load_trajectory = 0;
% image_save = 0;
% tri_view = 0;
% gpu_switch = 1;
% th_contact = 0.35;
% % time span
% init_cyc = 1;
% final_cyc = 160000;
% continue_running = init_cyc==1;

%% load or generate initial state
[xx,yy,zz] = meshgrid(1:N,1:N,1:N/2);
load('WorkSpace_Parameter.mat')
dr = Parameter(2)./110;
[final_cell_num,total_division_time,sim_name,div_time,div_axis,cell_size,h,tau,M,gamma,g,g_shell,c,eps,sigma_stack] = model_para(dr);
phi_stack = cell(final_cell_num,1);
phi_stack_new = cell(final_cell_num,1);
fphi_stack = cell(final_cell_num,1);
exp_stack = cell(final_cell_num,1);
phix_stack = cell(final_cell_num,1);
phiy_stack = cell(final_cell_num,1);
phiz_stack = cell(final_cell_num,1);
% image config %
if image_save == 1
	figure;set(gcf,'unit','normalized','position',[0,0,0.5,0.8]);
	if exist([sim_name '.avi'],'file')
		error('.avi file already exist !')
	else
		v = VideoWriter(sim_name);
		open(v)
		disp(['create ', sim_name,'.avi'])
	end
else
	v = [];
end
if init_cyc > 1 % continue running
	% time span %
	time_seq = init_cyc+1:final_cyc;
	% load T0 state %
	[phi_stack,div_count] = load_init_state(init_cyc,gpu_switch);
% 	[phi_stack,~,~,~,div_count] = cell_division_all(phi_stack,init_cyc,div_time,div_axis,cell_size,total_division_time,div_count);
	% trajectory %
	if trajectory_save>0
		if load_trajectory>0
			if exist([sim_name,'_trajectory.mat'],'file')
				load([sim_name,'_trajectory.mat'])
				fprintf('load %s_trajectory.mat\n',sim_name)
			else 
				warning('%s_trajectory.mat do not exist\n',sim_name)
				trajectory = cell(final_cell_num,total_division_time+1);
				fprintf('create %s_trajectory.mat\n',sim_name)
			end
		else
			if exist([sim_name,'_trajectory.mat'],'file')
				error('trajectory file already exist !')
			else
				trajectory = cell(final_cell_num,total_division_time+1);
				fprintf('create %s_trajectory.mat\n',sim_name)
			end
		end
		[v_mean,dvdt] = dynamics_anal_init(total_division_time,time_seq,div_count,div_time);
	end
else % new simulation
	% time span %
	time_seq = init_cyc:final_cyc;
	% T0 state %
	div_count = 1;
	r_cell = 106.5;
	if compressed == 1
		cell0_shape = shape_generate(r_cell,xx,yy,zz,N,[0.67,0.45],1,0.36);
	else
		cell0_shape = shape_generate(r_cell,xx,yy,zz,N,[0.534,0.534],0);
	end
	phi_cell0 = (tanh(cell0_shape/eps)+1)/2;
	if gpu_switch == 1
		phi_cell0 = gpuArray(phi_cell0);
	end
	phi_stack{1} = phi_cell0;
	% trajectory %
	if trajectory_save>0
		trajectory = cell(final_cell_num,total_division_time+1);
		fprintf('create %s_trajectory.mat\n',sim_name)
	end
end
phase_num = sum(~cellfun(@isempty,phi_stack));
% generate eggshell %
r_shell = 112;
if compressed == 1
	shell_shape = shape_generate(r_shell,xx,yy,zz,N,[0.67,0.45],1,0.36);
else
	shell_shape = shape_generate(r_shell,xx,yy,zz,N,[0.534,0.534],0);
end
phi_shell = (1-tanh(shell_shape/eps))/2;
check_eggshell(phi_stack,phi_shell,th_contact)
% init var %
L = dr*256;
k=(2*pi/L)*[0:(N/2-1) (-N/2):(-1)];
k2 = (2*pi/L*2)*[0:(N/4-1) (-N/4):(-1)];
cplx=complex(0,1);
[kx,ky,kz]=meshgrid(k,k,k2);
if gpu_switch == 1
	phi_shell = gpuArray(phi_shell);
	kx = gpuArray(kx);
	ky = gpuArray(ky);
	kz = gpuArray(kz);
end
ksq=kx.^2+ky.^2+kz.^2;
stage_cyc = 1;% might be a bug while contiune_running
critical_cyc = false;
sigma = sigma_stack{div_count};
sigma1 = max(sigma(:));
old_cyc_str = [];
fprintf('simulation %s in progress\n\n',sim_name)

%% simulation
tic 
for t = time_seq
	cell_rep = zeros(size(phi_shell),'gpuArray');
	adh_x = zeros(size(phi_shell),'gpuArray');
	adh_y = zeros(size(phi_shell),'gpuArray');
	adh_z = zeros(size(phi_shell),'gpuArray');
	% fft %
	for n = 1:phase_num
		fphi_stack{n} = fftn(phi_stack{n});
		phix_stack{n}=real(ifftn(cplx*fphi_stack{n}.*kx));
		phiy_stack{n}=real(ifftn(cplx*fphi_stack{n}.*ky));
		phiz_stack{n}=real(ifftn(cplx*fphi_stack{n}.*kz));
		Fvol = M*(cell_size(n,div_count)*dr^3-sum(phi_stack{n}(:)*dr^3))*sqrt(phix_stack{n}.^2+phiy_stack{n}.^2+phiz_stack{n}.^2);
		exp_stack{n} = Fvol-g_shell*phi_shell.^2.*phi_stack{n};
		cell_rep = cell_rep + phi_stack{n}.^2;
		adh_x = adh_x + phix_stack{n};
		adh_y = adh_y + phiy_stack{n};
		adh_z = adh_z + phiz_stack{n};
	end
	
	% adhesion and repulsion %
	for n = 1:phase_num
		cell_adh_x = sigma1*adh_x;
		cell_adh_y = sigma1*adh_y;
		cell_adh_z = sigma1*adh_z;
		for m = find(sigma(n,:)~=sigma1)
			cell_adh_x = cell_adh_x - (sigma1-sigma(n,m))*phix_stack{m};
			cell_adh_y = cell_adh_y - (sigma1-sigma(n,m))*phiy_stack{m};
			cell_adh_z = cell_adh_z - (sigma1-sigma(n,m))*phiz_stack{m};
		end
		adh_rep = g*(cell_rep-phi_stack{n}.^2).*phi_stack{n}+cell_adh_x.*phix_stack{n}+cell_adh_y.*phiy_stack{n}+cell_adh_z.*phiz_stack{n};
		G1=2*(phi_stack{n}.*(1-phi_stack{n}).^2-phi_stack{n}.^2.*(1-phi_stack{n}));
		% semi-implicit method %
		phi_stack_new{n} = real(ifftn((fphi_stack{n}+h/tau*fftn(exp_stack{n}-adh_rep-c(n)*G1))./(1+h*gamma(n)/tau*ksq)));
	end
	% update phi %
	phi_stack = phi_stack_new;
	% division %
	[phi_stack,phase_num,div_event,near_div,div_count] = cell_division_all(phi_stack,t,div_time,div_axis,cell_size,total_division_time,div_count);
	if div_event == 1
		sigma = sigma_stack{div_count};
		sigma1 = max(sigma(:));
		stage_cyc = 1;
		div_event = 0;
	end
	
	% display and analysis %
	if mod(t,analysis_step) == 0
		if stage_cyc>1&&dynamics_analyse == 1
			phi_stack_old = phi_stack_cpu;
		end
		phi_stack_cpu = cellfun(@gather,phi_stack,'UniformOutput',false);
		if image_save>0
			multiphase_disp(phi_stack_cpu,t,v,th_contact,dr,tri_view,sigma)
		end
		if trajectory_save > 0
			trajectory = multiphase_trajectory(phi_stack_cpu,div_count,trajectory,xx,yy,zz);
			save(sprintf('%s_trajectory.mat',sim_name),'trajectory')
			if dynamics_analyse == 1
				critical_cyc = false;
				unempty = ~cellfun(@isempty,trajectory(:,div_count));
				center = cell2mat(cellfun(@(x)x(end,:),trajectory(unempty,div_count),'UniformOutput',false));
				if stage_cyc>1
					Vsq = sum((center-center_old).^2,2);
					v_mean{div_count}(stage_cyc) = sqrt(sum(Vsq)/size(trajectory,1));
				end
				center_old = center;
				if stage_cyc>2
					dvdt{div_count}(stage_cyc) = v_mean{div_count}(stage_cyc)-v_mean{div_count}(stage_cyc-1);
				end
				if stage_cyc>5
					critical_cyc = any(islocalmin(v_mean{div_count}(stage_cyc-2:stage_cyc))|islocalmax(v_mean{div_count}(stage_cyc-2:stage_cyc))|...
						islocalmin(dvdt{div_count}(stage_cyc-2:stage_cyc))|islocalmax(dvdt{div_count}(stage_cyc-2:stage_cyc)));
				end
			end
		end
		stage_cyc = stage_cyc+1;
		cyc_str = sprintf('%d/%d',t,final_cyc);
		fprintf(repmat('\b',1,length(old_cyc_str)+1))
		disp(cyc_str)
		old_cyc_str = cyc_str;
	end
	if phi_save_switch > 0
		if critical_cyc
			phi_save = phi_stack_old;
			save_t = t-100;
			fprintf(repmat('\b',1,length(old_cyc_str)+1))
			fprintf('cyc-%d saved\n%s\n',save_t,cyc_str)
			save(sprintf('%s_%d.mat',sim_name,save_t),'phi_save','div_count')
			critical_cyc = false;
		end
		if mod(t,phi_save_step) == 0
			phi_save = phi_stack_cpu;
			save_t = t;
			save(sprintf('%s_%d.mat',sim_name,save_t),'phi_save','div_count')
		end
		if near_div
			phi_save = cellfun(@gather,phi_stack,'UniformOutput',false);
			save_t = t;
			save(sprintf('%s_%d.mat',sim_name,save_t),'phi_save','div_count')
		end
	end
end
toc

%% close
if phi_save_switch>0
	phi_save = phi_stack_cpu;
	save(sprintf('%s_%d.mat',sim_name,t),'phi_save','div_count')
end
if image_save == 1
  close(v)
end

%% subfunctions
function [phi_stack_new,phase_num,div_event,near_div,div_count_new] = cell_division_all(phi_stack,t,div_time,div_axis,cell_size,total_division_time,div_count)
	phase_num = sum(~cellfun(@isempty,phi_stack));
	phi_stack_new = phi_stack;
	near_div = 0;
	div_event = 0;
	div_count_new = div_count;
	for n = 1:phase_num
		if t == div_time(n,div_count)&&div_count <= total_division_time
			vol_ratio = cell_size(phase_num+1,div_count+1)/cell_size(n,div_count+1);
			[phi_stack_new{phase_num+1},phi_stack_new{n}] = cell_division(phi_stack{n},vol_ratio,div_axis{n,div_count},eps);
			phase_num = phase_num+1;
			div_event = 1;
			near_div = 0;
			fprintf('cell division at %d\n',t)
		elseif t+1 == div_time(n,div_count)&&div_count <= total_division_time
			near_div = 1;
		end
	end
	if div_event == 1
		div_count_new = div_count_new+1;
	end
end
function [phi_stack,div_count] = load_init_state(init_cyc,gpu_switch)
	filename_char = ls;
	if ispc
		filename_str = string(filename_char);
		file_idx = contains(filename_str,['_',num2str(init_cyc),'.mat']);
		filename = filename_str(file_idx);
	elseif isunix
		filename_list = regexp(filename_char,'\s','split');
		filename_list(cellfun(@isempty,filename_list)) = [];
		file_idx = cellfun(@(x)contains(x,['_',num2str(init_cyc),'.mat']),filename_list);
		filename = filename_list{file_idx};
	end
	load(filename,'phi_save','div_count')
	fprintf('load %s\n',filename)
	if gpu_switch == 1
		phi_stack = cellfun(@gpuArray,phi_save,'UniformOutput',false);
	else
		phi_stack = phi_save;
	end
end
function [v,dvdt] = dynamics_anal_init(total_division_time,time_seq,div_count,division_time)
	v = cell(1,total_division_time+1);
	dvdt = cell(1,total_division_time+1);
	stage_init = time_seq(1);
	for n = div_count:total_division_time+1
		next_div = division_time(:,n);
		next_div = unique(next_div(next_div~=0));
		if n<=total_division_time
			stage_end = time_seq(find(time_seq == next_div,1));
			stage_end = stage_end-1;
		else
			stage_end = time_seq(end);
		end
		stage_length = length(ceil(stage_init/100)*100:100:stage_end);
		if stage_length>0
			v{n} = zeros(stage_length,1);
			dvdt{n} = zeros(stage_length,1);
		end
		stage_init = stage_end;
	end
end
function check_eggshell(phi_stack,phi_shell,th)
	phase_num = sum(~cellfun(@isempty,phi_stack));
	phase_sum = zeros(size(phi_stack{1}));
	for n = 1:phase_num
		phase_sum = phase_sum + phi_stack{n};
	end
	box_cell = regionprops(gather(phase_sum>th),'BoundingBox');
	box_shell = regionprops(1-phi_shell>0.5,'BoundingBox');
	if sum(abs(box_shell.BoundingBox(4:6) - box_cell.BoundingBox(4:6)))>30
		error('check eggshell...error')
	else
		disp('check eggshell...done')
	end
end
