function multiphase_disp(phi_stack,t,v,threshold,dr,tri_view,sigma)
%% config cell name, color and sequence to disp
phase_num = length(phi_stack) - sum(cellfun(@isempty,phi_stack));
colormap(jet)
switch phase_num
	case 12
    c_seq = [3.25,5,1.25,7,4.25,2.25,6,8,2.75,0.75,3.75,1.75];
    cellname = {'ABala';'ABalp';'ABara';'ABarp';'ABpla';'ABplp';'ABpra';'ABprp';'MS';'E';'C';'P3'};
    obj_seq = [5,12,3,9,6,14,4,10,13,8,11,7];
	case 8
    c_seq = [3,5,1,7,4,2,6,8];
    cellname = {'ABal';'ABar';'ABpl';'ABpr';'MS';'E';'C';'P3'};
    obj_seq = [8,5,10,6,9,4,7,3];
	case 7
    c_seq = [3,5,1,7.5,4,2,6];
    cellname = {'ABal';'ABar';'ABpl';'ABpr';'MS';'E';'P2'};
    obj_seq = [7,4,9,5,8,3,6];
	case 6
    c_seq = [3,5.5,1,7.5,4,2];
    cellname = {'ABal';'ABar';'ABpl';'ABpr';'EMS';'P2'};
    obj_seq = [6,3,8,4,7,5];
	case 4
    c_seq = [3.5,5.5,1.5,7.5];
    cellname = {'ABa';'ABp';'EMS';'P2'};
    obj_seq = [4,6,5,3];
	case 3
    c_seq = [3.5,6.5,1.5];
    cellname = {'ABa';'ABp';'P1'};
    obj_seq = [3,5,4];
	case 2
    c_seq = [2.5,6.5];
    cellname = {'AB';'P1'};
    obj_seq = [4,3];
	case 1
    c_seq = 4.5;
    cellname = {'P0'};
    obj_seq = 3;
end
cellname = cellfun(@(x)[' ',x],cellname,'UniformOutput',false);

%% iso-surface
x = dr*(-127:128);
y = dr*(-127:128);
z = dr*(-63:64);
[X,Y,Z] = meshgrid(x,y,z);
phase_num = length(phi_stack) - sum(cellfun(@isempty,phi_stack));
faces = cell(phase_num,1);
verts = cell(phase_num,1);
for n = 1:phase_num
    [faces{n},verts{n}]= isosurface(Y,X,Z,phi_stack{n},threshold);
end

%% imaging
y_range = [-30 30];
x_range = [-20 20];
z_range = [-15,15];
if tri_view == 1
	y0 = 0.2;
	x0 = 0.1;
	xw = [15,30,30]*0.75/45;
	yw = [20,20,15]*0.75/45;
	view_point = [0,-1,0;0,0,-1;1,0,0];
	axes_stack = cell(3,1);
	for view_idx = 1:3
		axes_stack{view_idx} = axes('Position',[x0,y0,xw(view_idx),yw(view_idx)]);
		multiphase_plot(faces,verts,c_seq);
		axis equal;
		xlim(x_range);
		ylim(y_range);
		zlim(z_range);
		grid off
		view(axes_stack{view_idx},view_point(view_idx,:))
		camlight(-20,40)
		y=ylabel({'\itx\rm / A-P Axis (\mum)'});set(y,'Fontname','arial','Fontsize',20);
		x=xlabel({'\itz\rm / D-V Axis (\mum)'});set(x,'Fontname','arial','Fontsize',20);
		z=zlabel({'\ity\rm / L-R Axis (\mum)'});set(z,'Fontname','arial','Fontsize',20);
		set(gca,'FontSize',20,'Fontname','arial','XAxisLocation','top');
		caxis([1 8])
		if view_idx<3
			if xw(view_idx) ~= xw(view_idx+1)
				x0 = x0+0.1+xw(view_idx);
			end
			if yw(view_idx) ~= yw(view_idx+1)
				y0 = y0+0.1+yw(view_idx);
			end
			camroll(90)
		end
	end
	if ~isempty(sigma)
		% adjacent matrix and sigma
		adjm_axes = adj_sigma_legend(0.1,y0,0.25,phi_stack,sigma,c_seq,cellname,phase_num);
	else
		h=findobj(gca);
		l = legend(h(obj_seq),cellname,'FontSize',17.5,'FontName','arial');
		%         set(l,'box','off','position',[0,0.76,0.5,0])
		set(l,'box','off','position',[0,0.76,0.425,0])
		caxis([1 8])
	end
	if ~isempty(t)
		ti = sgtitle(t,'FontSize',20,'Fontname','arial');
	end
else
	width = 0.32;
	gap = 0.05;
	y0 = 0.5-width/2;
	if ~isempty(sigma)
		axes_stack{1} = axes('Position',[width+gap*3.2,y0,1.5*width,width]);
	end
	multiphase_plot(faces,verts,c_seq);
	axis equal;
	xlim(x_range);
	ylim(y_range);
	zlim(z_range);
	grid off
	view([0,0,-1])
	camlight(-20,40)
	camroll(90)
	y=ylabel({'\itx\rm / A-P Axis (\mum)'});set(y,'Fontname','arial','Fontsize',20);
	x=xlabel({'\itz\rm / D-V Axis (\mum)'});set(x,'Fontname','arial','Fontsize',20);
	z=zlabel({'\ity\rm / L-R Axis (\mum)'});set(z,'Fontname','arial','Fontsize',20);
	set(gca,'FontSize',20,'Fontname','arial','XAxisLocation','top');
	if isempty(sigma)
		axes_stack{1} = gca;
		h=findobj(gca);
		l = legend(h(obj_seq),cellname,'Location','eastoutside','FontSize',17.5,'FontName','arial');
		set(l,'box','off')
		caxis([1 8])
	else
		% adjacent matrix and sigma
		adjm_axes = adj_sigma_legend(1.7*gap,y0,width,phi_stack,sigma,c_seq,cellname,phase_num);
	end
	if ~isempty(t)
		ti = title(t,'FontSize',20,'Fontname','arial');
	end
end


%% image save
if ~isempty(v)
	writeVideo(v,getframe(gcf))
	cellfun(@delete,axes_stack)
	delete(ti)
	if isempty(sigma)
		delete(l)
	else
		cellfun(@delete,adjm_axes)
	end
else
	pause(0.1)
end

function patch_stack = multiphase_plot(faces,verts,c_seq)
patch_stack = cell(length(verts),1);
for n = 1:length(verts)
	C = c_seq(n)*ones(length(verts{n}),1);
	patch_stack{n} = patch('Vertices',verts{n},'Faces',faces{n},'FaceVertexCData',C,'FaceColor','interp','EdgeColor','none','FaceAlpha',0.7);
end


function adjm_axes = adj_sigma_legend(x0,y0,yw,phi_stack,sigma,c_seq,cellname,phase_num)
%adj_m and sigma
adjm_axes = cell(2,1);
adjm_axes{1} = axes('Position',[x0,y0,yw,yw]);
adj_m(phi_stack,sigma)
dx = 0.2/6*phase_num;
xticks(-phase_num+0.5+dx:1:-0.5+dx);xticklabels(flipud(cellname));xtickangle(60);xlim([-phase_num,0])
yticks(-(phase_num-0.5):1:-0.5);yticklabels(flipud(cellname));ylim([-phase_num,0])
if phase_num >= 12
	fontsize = 12;
else
	fontsize = 15;
end
set(gca,'FontSize',fontsize,'Fontname','arial')
% legend
adjm_axes{2} = axes('Position',[0.01,y0,0.02/0.25*yw,yw]);
c_idx = sort(c_seq);
f = 1:4;
w = 3*6/max([3,phase_num]);
for n = 1:phase_num
	vert = [[0;1;1;0],[0;0;1;1]-n];
	C = c_idx(n)*ones(length(vert),1);
	patch('Faces',f,'Vertices',vert,'FaceVertexCData',C,'FaceColor','interp','EdgeColor',[1,1,1],'LineWidth',w)
end
set(gca,'xticklabels',[],'yticklabels',[])
caxis([1 8])